/*
 * Copyright (c) 2001-2003 The Trustees of Indiana University.  
 *                         All rights reserved.
 * Copyright (c) 1998-2001 University of Notre Dame. 
 *                         All rights reserved.
 * Copyright (c) 1994-1998 The Ohio State University.  
 *                         All rights reserved.
 * 
 * This file is part of the LAM/MPI software package.  For license
 * information, see the LICENSE file in the top level directory of the
 * LAM/MPI source distribution.
 * 
 * $HEADER$
 *
 * $Id: 40_getvaluelen_cxx.cc,v 1.4 2003/05/04 13:47:18 jsquyres Exp $
 */

#include <stdio.h>
#include <string.h>
#include <mpi.h>

#include "lamtest_error.h"


static void test_info(MPI::Info& info, const char* key, const char* value);


int 
main(int argc, char** argv)
{
  const char *key1 = "key1";
  const char *key2 = "key2key2";
  const char *key3 = "key3key3key3";
  const char *value1 = "value1";
  const char *value2 = "value2 value2";
  const char *value3 = "value3 value3 value3";

  MPI::Init(argc, argv);

  /* Pretty simple test -- call MPI_Info_set followed by MPI_Info_get
     and compare that the value that we get out is the same as the one
     that we put in */

  MPI::Info info1, info2, info3;
  test_info(info1, key1, value1);
  test_info(info2, key2, value2);
  test_info(info3, key3, value3);

  /* Free them so that we can be bcheck clean */

  info1.Free();
  info2.Free();
  info3.Free();

  MPI::Finalize();
  return 0;
}


/*
 * Abort on failure
 */
static void
test_info(MPI::Info& info, const char* key, const char* value)
{
  int len = 0;
  int expected = -73;

  info = MPI::Info::Create();
  info.Set(key, value);
  if (!info.Get_valuelen(key, len) ||
      strlen(value) != len)
    lamtest_error(__FILE__, __LINE__, 
		  "Info::Get_valuelen obtained value %d, expected %d\n", 
		  len, strlen(value));

  len = expected;
  if (info.Get_valuelen("bogus", len))
    lamtest_error(__FILE__, __LINE__, 
		  "Info::Get_valuelen for bogus key reported flag==1\n");
  else if (len != expected)
    lamtest_error(__FILE__, __LINE__, 
		  "Info::Get_valuelen for bogus key reset len; "
		  "got %d, expected %d\n", len, expected);
}
